/*
 *  ZUC: Zaurus Unit Converter
 *  Copyright (C) 2002 Nicholas J Kreucher
 *
 *  This file is part of ZUC.
 *  
 *  ZUC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  ZUC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with ZUC; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 */

#include <qpe/qpeapplication.h>
#include "conv_matrix.h"
#include <qstring.h>
#include <qfile.h>
#include <qmap.h>
#include <qcombobox.h>
#include <qpopupmenu.h>
#include <iostream.h>
#include <stdlib.h>


conv_matrix::conv_matrix(char *s)
{
	QString tr;
#ifdef QPE
	QString uf(getenv("QPEDIR"));
	uf.append("/etc/zuc_units");

	file = new QFile(uf);
#else
	file = new QFile("/usr/share/zuc/zuc_units");
#endif

	if( !file->open(IO_ReadOnly) )
	  qWarning(tr.sprintf("conv_matrix::conv_matrix: could not open units file"));

	sectionAt = -1;
	setSection(s);
}
	

conv_matrix::~conv_matrix()
{
	delete file;
}


/*
 * lookup linear conversion
 */
long double conv_matrix::lookup(QString *f, QString *t)
{
	long double ret;
	QString tr;

	if( !basetoMap.contains(*f) || !basetoMap.contains(*t) )
	{
		qWarning("conv_matrix::lookup: unit conversion not found");
		return -1;
	}

	if( *f == *base )
		ret = basetoMap[t->latin1()];
	else if( *t == *base )
		ret = 1/basetoMap[f->latin1()];
	else
		ret = (1/basetoMap[f->latin1()] * basetoMap[t->latin1()]);

	return ret;
}

/*
 * set's (and does a file seek) to "section", or if already set to
 * this section, sets the file pointer to the start of "section"
 */
bool conv_matrix::setSection(char *s)
{
	QString tr, line;

	if( s == 0L && sectionAt != -1 )
	{
		file->at(sectionAt);
		return true;
	}
	else if( s == 0L )
	{
		section = 0L;
		return false;
	}

	if( section )
		delete section;	
	section = new QString(s);

	// don't need to do anything else for temperature
	if( *section == qApp->translate("@default", "temperature") )
		return true;

	file->at(0);

	/* sequencially search for "section". defn:
	 * 		section\s{\s\n\s<from>:<to>:<factor>\n ...\s}\n
	 */
	while( file->readLine(line, 64) != -1 )
	{
		stripcomments(&line);
		
		if( line.find(QRegExp(tr.sprintf("^%s:[/a-zA-Z0-9\\^]*[\\s]*[{$]", section->latin1()))) == 0 )
		{
			sectionAt = file->at();

			/* get the base */
			base = new QString(line);
			base->remove(0, base->find(':')+1);
			base->remove(base->find(QRegExp("\\s")),
				       	base->length()-1);
			qDebug(tr.sprintf("conv_matrix::setSection: found %s "\
						"(%s) at %d.",
						section->latin1(),
						base->latin1(),
					       	sectionAt));
			return true;
		}
		else if( line.stripWhiteSpace().right(1) == "{" )
		{
			if( !findSectionEnd() ) {
				qDebug("conv_matrix::setSection: } not found");
				return false;
			}
		}
	}

	qDebug(tr.sprintf("conv_matrix::setSection: section %s not found.",
				section->latin1()));
	delete section;
	sectionAt = -1;
	return false;
}


/*
 * find the section end, searches for exactly "}"
 */
bool conv_matrix::findSectionEnd()
{
	QString line;

	// get all the lines in the current section
	while( next_line(&line) );

	if( file->atEnd() )
		return false;

	return true;
}


/*
 * loads the list of sections from the file
 */
int conv_matrix::loadSections(QPopupMenu *menu)
{
	QString line, tr;
	int count = 0;

	while( file->readLine(line, 64) != -1 )
	{
		stripcomments(&line);
		
		if( line.find(QRegExp("^[a-zA-Z ]+:[/a-zA-Z0-9\\^]*[\\s]*[{$]")) == 0 )
		{
			if( !findSectionEnd() )
				return count;

			line.remove(line.find(':'), line.length()-1);
	
			qDebug(tr.sprintf("UNITS: found %s", line.latin1()));	
			menu->insertItem(line);
			count++;
		}
	}

	return count;
}


/*
 * gets the next line from the file
 * (todo: for the current category)
 */
bool conv_matrix::next_line(QString *line)
{	
	if( file->readLine(*line, 64) == -1 || file->atEnd()
	 	|| line->stripWhiteSpace() == "}" )
	{
		stripcomments(line);

		// rewind the section to next call so "next_line" loops around
		//setSection();
		return false;
	}

	stripcomments(line);
	
	return true;
}


/*
 * strips comments from a QString
 */
void conv_matrix::stripcomments(QString *line)
{
	int i;
	
	if( (i = line->find('#')) != -1 )
		line->truncate(i);
}

	
/*
 * inserts aviabable conversion factors into "from" and "to" QComboBox(s)
 * (todo: and fills the converstion matrix for the category)
 */
int conv_matrix::populate(QComboBox *fromBox, QComboBox *toBox)
{
	QString line, copy, tr;
	long double factor;
	
	if( !setSection() )
		return 0;

	basetoMap.clear();
	fromBox->clear();
	toBox->clear();

	while( next_line(&line) )
	{
		line = line.stripWhiteSpace();
		// line is <from>:<to>:<factor>
		// first get the factor, preserving line for <to> and <from>
		copy = line;
		copy.remove(0, copy.findRev(':')+1);
		factor = copy.toDouble();
		

		// gets the <from>, preserving line to get <to> later
		copy = line;
		copy.remove(copy.find(':'),copy.length()-1);
		if( copy != *base ) {
			basetoMap.insert(copy, 1/factor);
			qDebug(tr.sprintf("MAP ins: %s->%s:%.15lf", base->latin1(),
				copy.latin1(), (double) 1/ (double) factor));
		}
		
		// gets the <to>, destroying line
		line.remove(0, line.find(':')+1);
	       	line.remove(line.findRev(':'),line.length()-1);
		if( line != *base ) {
			basetoMap.insert(line, factor);
			qDebug(tr.sprintf("MAP ins: %s->%s:%.15lf", base->latin1(),
						line.latin1(), (double) factor));
		}
	
	}

	basetoMap.insert(*base, 1.0);

	stringlongMap::Iterator it;
	for( it = basetoMap.begin(); it != basetoMap.end(); it++)
	{
		fromBox->insertItem(it.key());
		toBox->insertItem(it.key());
	}

	return basetoMap.count();
}


/* populates the conv from and to boxes with temperature choices */
void conv_matrix::loadTemperature(QComboBox *fromBox, QComboBox *toBox)
{
	char *temps[] = {"Kelvin","Celsius","Fahrenheit"}; //,"Reaumur","rankine"};
	short i = 0;
	
	basetoMap.clear();
	fromBox->clear();
	toBox->clear();

	for(i = 0; i < 3; i++) {
		fromBox->insertItem(temps[i]);
		toBox->insertItem(temps[i]);
	}
}


/* handle's dymanic conversion */
long double conv_matrix::convert(QString *f, long double fval,
		QString *t)
{
	if( *section == qApp->translate("@default", "temperature") )
		return convTemperature(f, fval, t);

	return lookup(f, t) * fval;
}


// handle's the temperature scale
long double conv_matrix::convTemperature(QString *f, long double fval,
		QString *t)
{
	const long double kel = 273.15;
	//const long double rak = 459.6;
	long double from = 0;

	// if kelvin is not in there, convert it to kelvin first
	if( *f != "Kelvin" && *t != "Kelvin" )
	{
		QString* szkel = new QString("Kelvin");
		from = convTemperature(f, fval, szkel);
		delete szkel;
		
		f = new QString("Kelvin");
	}
	else
		from = fval;

	// from kelvin to XXXX
	if( *f == "Kelvin" )
	{
		switch(t->latin1()[0])
		{
			case 'k':
				return from;

			case 'C':
				return from-kel;

			case 'F':
				return (from-kel)*((long double) 9/5) + 32;

		}
	}

	// from XXXX to kelvin
	if( *t == "Kelvin" )
	{
		switch(f->latin1()[0])
		{
			case 'k':
				return from;

			case 'C':
				return from+kel;

			case 'F':
				return (from-32)*((long double) 5/9) + kel;
		}
						
	}

	return 0;
}

