/*
 *  ZUC: Zaurus Unit Converter
 *  Copyright (C) 2002 Nicholas J Kreucher
 *
 *  This file is part of ZUC.
 *  
 *  ZUC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  ZUC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with ZUC; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 */
#include "zuc.h"
#include "conv_matrix.h"
#include "unitHistMenu.h"
#include "zuc_totd.h"
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qclipboard.h>
#include <qcursor.h>
#include <qtoolbutton.h>
#include <qpe/config.h>
#include <stdlib.h>

/* 
 *  Constructs a zuc which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
zuc::zuc( QWidget* parent,  const char* name, WFlags fl )
    : zucui( parent, name, fl )
{
	Config *cf;
	QString tmp;

	QPopupMenu *help_menu = new QPopupMenu(this, name);
#ifdef QPE
	zuc_mb = new QPEMenuBar(this, name);
#elif KDE
	zuc_mb = new KMenuBar(this, name);
#else
	zuc_mb = new QMenuBar(this, name);
#endif
	units_menu = new QPopupMenu(this, name);
	copy_menu = new QPopupMenu(this, name);
	unitHist_menu = new unitHistMenu(this, name);
	precision_menu = new QPopupMenu(this, name);
	display_menu = new QPopupMenu(this, name);

	/* setup conversion matrix */
	convit = new conv_matrix();
	
	/* setup Units menu */
	units_menu->setCheckable(true);
	units_menu->insertTearOffHandle(-1, 0);
	units_menu->setCaption(tr("Units"));
	convit->loadSections(units_menu);
	/* load temperature (non-linear) manually */
	tempid = units_menu->insertItem(tr("temperature"));
	connect(units_menu, SIGNAL(activated(int)),
			this, SLOT(switchSection(int)));

	/* setup Help menu */
	help_menu->insertItem(tr("&Tip of the Day"), this, SLOT(totd_go()));
	help_menu->insertSeparator();
	help_menu->insertItem(tr("&About"), this, SLOT(about()), CTRL+Key_H);
	help_menu->insertItem(tr("About &Qt"), this, SLOT(aboutQt()));

	/* precision menu */
	precision_menu->setCheckable(true);
	for(int i = 0; i <= 10; i++)
		precision_menu->insertItem(tmp.sprintf("%d", i));	
	connect(precision_menu, SIGNAL(activated(int)),
			this, SLOT(setPrecision(int)));
	
	/* setup Display menu */
	display_menu->setCheckable(true);
	/*
	display_menu->insertItem("&Standard Notation");
	display_menu->insertItem("&Scientific Notation");
	display_menu->setItemEnabled(display_menu->idAt(1), false);
	display_menu->insertItem("&Engineering Notation");
	display_menu->setItemEnabled(display_menu->idAt(2), false); */
	idSwap = display_menu->insertItem(tr("Show &Swap Button"));
	connect(display_menu, SIGNAL(activated(int)),
			this, SLOT(displayMenu(int)));
	display_menu->insertSeparator();
	display_menu->insertItem(tr("&Precision"), precision_menu);
	
	/* setup menu bar */
	zuc_mb->insertItem(tr("&Units"), units_menu);
	zuc_mb->insertItem(tr("&Display"), display_menu);
	zuc_mb->insertSeparator();
	zuc_mb->insertItem(tr("&Help"), help_menu);
	zuc_mb->setSeparator(QMenuBar::InWindowsStyle);
	zuc_mb->show();

	/* setup copy popup menu */
	copy_menu->insertItem(tr("Copy result with unit"), this,
			SLOT(copyToClipboard()));
	copy_menu->insertItem(tr("Copy result without unit"), this,
			SLOT(copyToClipboard_nosz()));
	copyBtn->setPopup(copy_menu);

	/* setup unit history popup menu */
	unitHistBtn->setPopup(unitHist_menu);
	connect(unitHist_menu, SIGNAL(activated(int)),
			this, SLOT(setUnitsFromHist(int)));
	
	/* set default section and load the lists into to/from drop downs */
	result = 0;
	cf = new Config("ZUC");
	cf->setGroup("config");
	switchSection(units_menu->idAt(cf->readNumEntry("units_menu", 1)));
	conv_fromBox->setCurrentItem(cf->readNumEntry("fromBox", 0));
	conv_toBox->setCurrentItem(cf->readNumEntry("toBox", 0));
	setPrecision(precision_menu->idAt(cf->readNumEntry("precision", 3)));
	cf->readNumEntry("swapBtn", 1) ? swapBtn->hide() : swapBtn->show();
	displayMenu(idSwap);
	
	/* load history */
	cf->setGroup("history");

	int max = cf->readNumEntry("count", 0);
	int i = 0;
	
	while( i < max )
	{
		tmp.sprintf("from%d", i);
		from_resultList->insertItem(cf->readEntry(tmp));
		tmp.sprintf("to%d", i);
		to_resultList->insertItem(cf->readEntry(tmp));

		i++;
	}

	delete cf;
	
	/* show tip of the day 
	totd_go(); */
}

/*  
 *  Destroys the object and frees any allocated resources
 */
zuc::~zuc()
{
	Config *cf;
	int i;
	QString tmp;
	
	if( convit )
		delete convit;

	/* save configuration state */
	cf = new Config("ZUC");
	cf->setGroup("config");

	/* i-- > 2 as 0 is the seperator, therefore default is 1 */
	i = units_menu->count() - 1;
	while( (units_menu->isItemChecked(units_menu->idAt(i)) == false) && (i-- > 2) );
	cf->writeEntry("units_menu", i);
	cf->writeEntry("fromBox", conv_fromBox->currentItem());
	cf->writeEntry("toBox", conv_toBox->currentItem());
	cf->writeEntry("precision", precision);
	cf->writeEntry("swapBtn", (int) !swapBtn->isHidden());
	
	/* save history */
	cf->setGroup("history");

	i = from_resultList->count();
	i = (i <= 10) ? i : 25;
	cf->writeEntry("count", i);
	
	i--;
	while( i >= 0 )
	{
		tmp.sprintf("from%d", i);
		cf->writeEntry(tmp, from_resultList->text(i));
		tmp.sprintf("to%d", i);
		cf->writeEntry(tmp, to_resultList->text(i));

		i--;
	}

	delete cf;
}


/*
 *  the close slot
 */
void zuc::goodBye()
{
    close();
}

/*
 * convert the value!
 */
void zuc::do_conv_go()
{
	long double conv_num;
	QString from_result, to_result, tr, section;
	bool ok;
	
	from_result = conv_fromBox->currentText();
	to_result = conv_toBox->currentText();

	// grab the text (hopefully number) from the conv_meBox
	// and do the conversion
	conv_num = conv_meBox->text().stripWhiteSpace().toDouble(&ok);
	if( ok )
       	{
		// append this to the unit history
		section = convit->getSection();
		unitHist_menu->addHist(from_result, to_result, section);

		// convert the value
		result = convit->convert(&from_result, conv_num, &to_result);

		// display results in the big LCD displays
		fromLabel->setText(from_result);
		fromDpy->display((double) toPrecision(conv_num));
	
		restoreLCD();
		toLabel->setText(to_result);
		toDpy->display((double) toPrecision(result));
		
		// and put the numbers in the results listing tab
		from_result.insert(0, tr.sprintf("%.*lf ", precision, (double) conv_num));
		from_resultList->insertItem(from_result, 0);
		
		to_result.insert(0, tr.sprintf("%.*lf ", precision, (double) result));
		to_resultList->insertItem(to_result, 0);
	}
}


/*
 * convert the result to the selected precision (for LCD display)
 */
long double zuc::toPrecision(long double val)
{
	QString tmp;
	
	tmp.sprintf("%.*lf", precision, (double) val);
	return (long double) tmp.toDouble();
}


/*
 * display menu callback
 */
void zuc::displayMenu(int id)
{
	/* show/hide swap button */
	if( id == idSwap )
	{
		if( swapBtn->isHidden() )
		{
			/* unhide */
			swapBtn->show();
			display_menu->setItemChecked(idSwap, true);
			conv_fromBox->setGeometry(50, 60, 150, 20);
			conv_toBox->setGeometry(50, 80, 150, 20);
		}
		else
		{
			/* hide */
			swapBtn->hide();
			display_menu->setItemChecked(idSwap, false);
			conv_fromBox->setGeometry(40, 60, 160, 20);
			conv_toBox->setGeometry(40, 80, 160, 20);

		}
	}
}


/*
 * from menubar: switch section
 */
void zuc::switchSection(const char *secsz)
{
	QString tmp(tr("temperature"));

	// don't reload a section if we are already in it
	if( convit->getSection() == secsz )
		return;
	
	// see if it's temperature
	if( tmp == secsz && convit->setSection((char*) secsz) )
		convit->loadTemperature(conv_fromBox, conv_toBox);
	else if( convit->setSection((char*) secsz ) )
		convit->populate(conv_fromBox, conv_toBox);
	else
		return;
	
	for(unsigned int i = 0; i < units_menu->count(); i++)
	{
		if( units_menu->text(units_menu->idAt(i)) != secsz )
			units_menu->setItemChecked(units_menu->idAt(i), false);
		else
			units_menu->setItemChecked(units_menu->idAt(i), true);
	}
}


/*
 * call same function, but with id not string
 */
void zuc::switchSection(int sec)
{
	switchSection( units_menu->text(sec).latin1() );
}



/*
 *  change sig figs for display
 */
void zuc::setPrecision(int id)
{
	QString tmp;

	// check new item
	for(unsigned int i = 0; i < precision_menu->count(); i++)
		precision_menu->setItemChecked(precision_menu->idAt(i), false);

	precision_menu->setItemChecked(id, true);

	if( precision_menu->text(id) == tr("max") )
		precision = 10;
	else
		precision = atoi(precision_menu->text(id));
}


/*
 * unit pair from the unitHistBtn menu was choosen
 */
void zuc::setUnitsFromHist(int id)
{
	int i;
	const char *from = unitHist_menu->getFromUnit(unitHist_menu->indexOf(id));
	const char *to = unitHist_menu->getToUnit(unitHist_menu->indexOf(id));

	switchSection( unitHist_menu->getSection(unitHist_menu->indexOf(id)) );

	i = 0;
	while( conv_fromBox->text(i) != from && i < conv_fromBox->count() )
		i++;
	if( i < conv_fromBox->count() )
		conv_fromBox->setCurrentItem(i);

	i = 0;
	while( conv_toBox->text(i) != to && i < conv_toBox->count() )
		i++;
	if( i < conv_toBox->count() )
		conv_toBox->setCurrentItem(i);
}


/*
 * copies the value from the listbox item to the input box
 */
void zuc::send_to_conv_me(QListBoxItem *item)
{
	parsePaste(item->text().stripWhiteSpace());
}


/*
 * takes a string and puts it in the input box, also
 * trying to set the apperoperte unit
 */
void zuc::parsePaste(QString num)
{
	QString unit, tr;
	int i;

	unit = num;
	
	num.remove(num.find(' '), num.length()-1);
	unit.remove(0, unit.find(' ')+1);
	
	conv_meBox->setText(num);

	if( unit.length() == 0 )
		return;

	i = 0;
	while( conv_fromBox->text(i) != unit && i < conv_fromBox->count() )
		i++;

	if( i < conv_fromBox->count() )
		conv_fromBox->setCurrentItem(i);
}


/*
 * about dialog slot
 */
void zuc::about()
{
	QMessageBox::about(this, tr(ZUCNAME),
		tr("The Zaurus Unit Converter is a\nutility to convert any value\nfrom one unit to another.\n\nVersion: %1\nBuild date: %2\n\nAuthor: Nicholas J Kreucher\nE-Mail: <nick@kreucher.net>\n\nhttp://zuc.sourceforge.net/\n\nLicense: GNU Public License").arg(VERSION).arg(BUILDTIME));
}


/*
 * about Qt dialog slot
 */
void zuc::aboutQt()
{
	QMessageBox::aboutQt(this, tr(ZUCNAME));
}


/*
 * tip of the day dialog box
 */
void zuc::totd_go()
{
	qDebug("TOTD Slot");
	zuc_totd totd_box(this, "ZUC", true);
	
	totd_box.exec();	
}


/*
 * lcd overflow
 */
void zuc::lcd_overflow()
{
	qDebug("LCD Overflow!");
	int numDigits;

	switch( QMessageBox::warning( this, tr(ZUCNAME),
				tr("Result to large for LCD panel.\nReduce size of LCD numbers to\naccomidate?"),
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No | QMessageBox::Escape ))
       	{
		case QMessageBox::Yes:
			toDpy->setSmallDecimalPoint(true);
			numDigits = toDpy->numDigits();

			while( toDpy->checkOverflow((double) result) &&
					numDigits < 30 )
			{
				toDpy->setNumDigits(++numDigits);
			}

			if( numDigits == 30 )
			{
				restoreLCD(QLCDNumber::DEC, true);
			}
			else
				toDpy->display((double) result);
			break;
		
		case QMessageBox::No:
			restoreLCD(QLCDNumber::DEC, true);
			break;
	}
}


/*
 * retore the LCD results panel to defaults (for us)
 */
void zuc::restoreLCD( QLCDNumber::Mode base, bool setBase )
{
	if( toDpy->numDigits() != 9 )
		toDpy->setNumDigits(9);

	if( toDpy->smallDecimalPoint() )
		toDpy->setSmallDecimalPoint(false);

	if( setBase )
	{
		toDpy->setMode(base);
		switch( base )
		{
			case QLCDNumber::DEC:
				decRB->setChecked(true);
				break;

			case QLCDNumber::HEX:
				hexRB->setChecked(true);
				break;

			case QLCDNumber::OCT:
				octRB->setChecked(true);
				break;

			case QLCDNumber::BIN:
				binRB->setChecked(true);
				break;
		}
	}
}


/*
 * copy's current result (or selected item in listing) to system clipboard
 */
void zuc::copyToClipboard()
{
	QString szResult, tr;

#ifdef QPE	
	QClipboard *cb = QPEApplication::clipboard();
#else
	QClipboard *cb = QApplication::clipboard();
#endif

	szResult.sprintf("%.*lf %s", precision, (double) result, toLabel->text().latin1());
	tr.sprintf("CB: copying %s to system clipboard", szResult.latin1());
	qDebug(tr);
	cb->setText(szResult);
}


/*
 * same as above, w/o the string
 */
void zuc::copyToClipboard_nosz()
{
	QString szResult, tr;
	
#ifdef QPE	
	QClipboard *cb = QPEApplication::clipboard();
#else
	QClipboard *cb = QApplication::clipboard();
#endif

	szResult.sprintf("%.*lf", precision, (double) result);
	tr.sprintf("CB: copying %s to system clipboard", szResult.latin1());
	qDebug(tr);
	cb->setText(szResult);
}


/*
 * paste's the system clipboard into the conv_meBox
 */
void zuc::pasteFromClipboard()
{
	QString tr;

#ifdef QPE	
	QClipboard *cb = QPEApplication::clipboard();
#else
	QClipboard *cb = QApplication::clipboard();
#endif

	tr.sprintf("CB: pasting %s from system clipboard", cb->text().latin1());
	qDebug(tr);

	parsePaste(cb->text());
}


/*
 * swap to and from comboboxes
 */
void zuc::swap_tofrom()
{
	int tmp;

	tmp = conv_fromBox->currentItem();
	conv_fromBox->setCurrentItem(conv_toBox->currentItem());
	conv_toBox->setCurrentItem(tmp);
}

